@@ -4,7 +4,7 @@
 
 /* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015
  * License: GPL
- * Copyright (c) The Exim Maintainers 2015 - 2018
+ * Copyright (c) The Exim Maintainers 2015 - 2020
  */
 
 /* Code for calling virus (malware) scanners. Called from acl.c. */
@@ -109,7 +109,7 @@ features_malware(void)
 {
 const uschar * s;
 uschar * t;
-uschar buf[64];
+uschar buf[EXIM_DRIVERNAME_MAX];
 
 spf(buf, sizeof(buf), US"_HAVE_MALWARE_");
 
@@ -262,6 +262,7 @@ static inline int
 m_panic_defer_3(struct scan * scanent, const uschar * hostport,
   const uschar * str, int fd_to_close)
 {
+DEBUG(D_acl) debug_print_socket(fd_to_close);
 (void) close(fd_to_close);
 return m_panic_defer(scanent, hostport, str);
 }
@@ -302,8 +303,7 @@ const uschar * rerror;
 int roffset;
 const pcre * cre;
 
-cre = pcre_compile(CS re, PCRE_COPT, (const char **)&rerror, &roffset, NULL);
-if (!cre)
+if (!(cre = pcre_compile(CS re, PCRE_COPT, CCSS &rerror, &roffset, NULL)))
   *errstr= string_sprintf("regular expression error in '%s': %s at offset %d",
       re, rerror, roffset);
 return cre;
@@ -317,7 +317,7 @@ int i = pcre_exec(cre, NULL, CS text, Ustrlen(text), 0, 0,
 	      ovector, nelem(ovector));
 uschar * substr = NULL;
 if (i >= 2)				/* Got it */
-  pcre_get_substring(CS text, ovector, i, 1, (const char **) &substr);
+  pcre_get_substring(CS text, ovector, i, 1, CCSS &substr);
 return substr;
 }
 
@@ -339,6 +339,7 @@ else
 return cre;
 }
 
+
 /*
  Simple though inefficient wrapper for reading a line.  Drop CRs and the
  trailing newline. Can return early on buffer full. Null-terminate.
@@ -370,8 +371,12 @@ while ((rcv = read(fd, p, 1)) > 0)
   }
 if (!ok)
   {
-  DEBUG(D_acl) debug_printf_indent("Malware scan: read %s (%s)\n",
+  DEBUG(D_acl)
+    {
+    debug_printf_indent("Malware scan: read %s (%s)\n",
 		rcv==0 ? "EOF" : "error", strerror(errno));
+    debug_print_socket(fd);
+    }
   return rcv==0 ? -1 : -2;
   }
 *p = '\0';
@@ -655,11 +660,11 @@ if (!malware_ok)
     switch(scanent->conn)
     {
     case MC_TCP:
-      malware_daemon_ctx.sock = ip_tcpsocket(scanner_options, &errstr, 5);	break;
+      malware_daemon_ctx.sock = ip_tcpsocket(scanner_options, &errstr, 5, NULL); break;
     case MC_UNIX:
       malware_daemon_ctx.sock = ip_unixsocket(scanner_options, &errstr);	break;
     case MC_STRM:
-      malware_daemon_ctx.sock = ip_streamsocket(scanner_options, &errstr, 5);	break;
+      malware_daemon_ctx.sock = ip_streamsocket(scanner_options, &errstr, 5, NULL); break;
     default:
       /* compiler quietening */ break;
     }
@@ -792,7 +797,7 @@ if (!malware_ok)
       if (*scanner_options != '/')
 	{
 	/* calc file size */
-	if ((drweb_fd = open(CCS eml_filename, O_RDONLY)) == -1)
+	if ((drweb_fd = exim_open2(CCS eml_filename, O_RDONLY)) == -1)
 	  return m_panic_defer_3(scanent, NULL,
 	    string_sprintf("can't open spool file %s: %s",
 	      eml_filename, strerror(errno)),
@@ -1363,13 +1368,10 @@ badseek:  err = errno;
 	malware_name = US"unknown";
 
 	/* re-open the scanner output file, look for name match */
-	scanner_record = fopen(CS file_name, "rb");
-	while (fgets(CS linebuffer, sizeof(linebuffer), scanner_record))
-	  {
-	  /* try match */
-	  if ((s = m_pcre_exec(cmdline_regex_re, linebuffer)))
+	scanner_record = Ufopen(file_name, "rb");
+	while (Ufgets(linebuffer, sizeof(linebuffer), scanner_record))
+	  if ((s = m_pcre_exec(cmdline_regex_re, linebuffer))) /* try match */
 	    malware_name = s;
-	  }
 	(void)fclose(scanner_record);
 	}
       else /* no virus found */
@@ -1639,7 +1641,7 @@ badseek:  err = errno;
 	      malware_daemon_ctx.sock);
 
 	/* calc file size */
-	if ((clam_fd = open(CS eml_filename, O_RDONLY)) < 0)
+	if ((clam_fd = exim_open2(CS eml_filename, O_RDONLY)) < 0)
 	  {
 	  int err = errno;
 	  return m_panic_defer_3(scanent, NULL,
@@ -2280,12 +2282,13 @@ if (!fprot6d_re_virus)
 }
 
 
-void
-malware_show_supported(FILE * f)
+gstring *
+malware_show_supported(gstring * g)
 {
-fprintf(f, "Malware:");
-for (struct scan * sc = m_scans; sc->scancode != (scanner_t)-1; sc++) fprintf(f, " %s", sc->name);
-fprintf(f, "\n");
+g = string_cat(g, US"Malware:");
+for (struct scan * sc = m_scans; sc->scancode != (scanner_t)-1; sc++)
+  g = string_fmt_append(g, " %s", sc->name);
+return string_cat(g, US"\n");
 }
 
 
