@@ -5,7 +5,7 @@
 /* Experimental SPF support.
    Copyright (c) Tom Kistner <tom@duncanthrax.net> 2004
    License: GPL
-   Copyright (c) The Exim Maintainers 2016
+   Copyright (c) The Exim Maintainers 2016 - 2020
 */
 
 #ifdef SUPPORT_SPF
@@ -25,9 +25,11 @@ typedef struct spf_result_id {
 } spf_result_id;
 
 /* prototypes */
+void spf_lib_version_report(FILE *);
 BOOL spf_init(void);
 BOOL spf_conn_init(uschar *, uschar *);
 int  spf_process(const uschar **, uschar *, int);
+void spf_response_debug(SPF_response_t *);
 
 #define SPF_PROCESS_NORMAL  0
 #define SPF_PROCESS_GUESS   1
